// RGBColor.cpp
// Stores and manipulates rgb colors (no alpha)

// 	Copyright (C) Kevin Suffern 2000-2007.
//	This C++ code is for non-commercial purposes only.
//	This C++ code is licensed under the GNU General Public License Version 2.
//	See the file COPYING.txt for the full license.


// This file contains the definition of the class RGBColor

#include <math.h>    // for pow

#include "RGBColor.h"

// -------------------------------------------------------- default constructor

RGBColor::RGBColor(void)
	: r(0.0), g(0.0), b(0.0)
{}


// -------------------------------------------------------- constructor

RGBColor::RGBColor(float c)
	: r(c), g(c), b(c)
{}


// -------------------------------------------------------- constructor

RGBColor::RGBColor(float _r, float _g, float _b)
	: r(_r), g(_g), b(_b)
{}


// -------------------------------------------------------- copy constructor

RGBColor::RGBColor(const RGBColor& c)
	: r(c.r), g(c.g), b(c.b)
{}


// -------------------------------------------------------- destructor

RGBColor::~RGBColor(void)
{}


// --------------------------------------------------------assignment operator

RGBColor&
RGBColor::operator= (const RGBColor& rhs) {
	if (this == &rhs)
		return (*this);

	r = rhs.r; g = rhs.g; b = rhs.b;

	return (*this);
}


// -------------------------------------------------------- powc
// raise each component to the specified power
// used for color filtering in Chapter 28

RGBColor
RGBColor::powc(float p) const {
	return (RGBColor(pow(r, p), pow(g, p), pow(b, p)));
}

// -------------------------------------------------------- for printing with cout
ostream&
operator<<(ostream& os, const RGBColor& c )
{
    return os <<  "(" << c.r << "," << c.g << ","  << c.b << ")" << endl;
}




