// Sphere.cpp

#include "Sphere.h"

extern const double kEpsilon;
// ---------------------------------------------------------------- default constructor

Sphere::Sphere(void)
	: 	GeometricObject(),
		center(0.0),
		radius(1.0)
{}

// ---------------------------------------------------------------- constructor

Sphere::Sphere(Vector3D c, double r)
	: 	GeometricObject(),
		center(c),
		radius(r)
{}

// ---------------------------------------------------------------- clone

GeometricObject*
Sphere::clone(void) const {
	return (new Sphere(*this));
}

// ---------------------------------------------------------------- copy constructor

Sphere::Sphere (const Sphere& sphere)
	: 	GeometricObject(sphere),
		center(sphere.center),
		radius(sphere.radius)
{}

// ---------------------------------------------------------------- assignment operator

Sphere&
Sphere::operator= (const Sphere& rhs)
{
	if (this == &rhs)
		return (*this);

	GeometricObject::operator= (rhs);

	center 	= rhs.center;
	radius	= rhs.radius;

	return (*this);
}

// ---------------------------------------------------------------- destructor

Sphere::~Sphere(void) {}

//---------------------------------------------------------------- hit

ShadeRec
Sphere::hit(const Ray& ray) const {
	double 		t;
	Vector3D	temp 	= ray.o - center;
	double 		a 		= ray.d * ray.d;
	double 		b 		= 2.0 * temp * ray.d;
	double 		c 		= temp * temp - radius * radius;
	double 		disc	= b * b - 4.0 * a * c;

    ShadeRec sr;
    sr.hit_an_object = false;

    if (disc >= 0.0)
	{
		double e = sqrt(disc);
		double denom = 2.0 * a;
		t = (-b - e) / denom;    // smaller root

		if (t > kEpsilon) {
			sr.t = t;
			sr.normal 	 = (temp + t * ray.d) / radius;
			sr.local_hit_point = ray.o + t * ray.d;
			sr.hit_an_object = true;
			return sr;
		}

		t = (-b + e) / denom;    // larger root

		if (t > kEpsilon) {
			sr.t = t;
			sr.normal   = (temp + t * ray.d) / radius;
			sr.local_hit_point = ray.o + t * ray.d;
			sr.hit_an_object = true;
			return sr;
		}
	}

	return sr;
}




